local pClass = select(2, UnitClass("player"))
if pClass ~= "WARRIOR" and pClass ~= "PALADIN" and pClass ~= "DRUID" and pClass ~= "DEATHKNIGHT" and pClass ~= "MONK" then
	return
end

local PitBull4 = PitBull4
if not PitBull4 then
	error("PitBull4_VengeanceBar requires PitBull4")
end


local ShowVengeance, MaxVengeanceAP

local function IsTank()
	local spec = GetSpecialization()
	return GetSpecializationRole(spec) == "TANK"
end

local function IsInTankStance()
	if pClass == "PALADIN" then
		return true -- not sure how paladin vengeance works, seems awkward to tie it to a seal or RF, though
	elseif pClass == "DEATHKNIGHT" or pClass == "DRUID" then
		return GetShapeshiftForm() == 1
	elseif pClass == "WARRIOR" then
		return GetShapeshiftFormID() == 18
	elseif pClass == "MONK" then
		return GetShapeshiftFormID() == 23
	end
end

local GetVengeanceAP
do
	local vengeance = GetSpellInfo(93098)
	function GetVengeanceAP()
		local ap = select(15, UnitBuff("player", vengeance))
		if not ap then
			if not InCombatLockdown() then -- reset if we lose it out of combat
				MaxVengeanceAP = 1
			end
			return 0
		end
		MaxVengeanceAP = max(MaxVengeanceAP, ap)
		return ap
	end
end

local function GetVengeanceAPMax()
	return ShowVengeance and MaxVengeanceAP or 0
end

--Allow for getting the AP value in LuaTexts
local PitBull4_LuaTexts = PitBull4:GetModule("LuaTexts", true)
if PitBull4_LuaTexts then
	PitBull4_LuaTexts.ScriptEnv.VengeanceAP = GetVengeanceAP
	PitBull4_LuaTexts.ScriptEnv.VengeanceAPMax = GetVengeanceAPMax
end

local L = LibStub("AceLocale-3.0"):GetLocale("PitBull4_VengeanceBar")

local PitBull4_VengeanceBar = PitBull4:NewModule("VengeanceBar", "AceEvent-3.0")

PitBull4_VengeanceBar:SetModuleType("bar")
PitBull4_VengeanceBar:SetName(L["Vengeance bar"])
PitBull4_VengeanceBar:SetDescription(L["Show a bar indicating the attack power gained from your Vengeance."])
PitBull4_VengeanceBar:SetDefaults({
	position = 4,
	check_stance = false,
})

function PitBull4_VengeanceBar:OnEnable()
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	self:PLAYER_SPECIALIZATION_CHANGED()
end

function PitBull4_VengeanceBar:OnDisable()
	ShowVengeance = nil
end

function PitBull4_VengeanceBar:GetValue(frame)
	if not ShowVengeance or frame.unit ~= "player" then
		return nil
	end

	return GetVengeanceAP() / MaxVengeanceAP
end
function PitBull4_VengeanceBar:GetExampleValue(frame)
	if frame.unit == "player" then
		return 0.6
	end
end

function PitBull4_VengeanceBar:GetColor()
	local color = PitBull4.PowerColors.POWER_TYPE_HEAT
	if color then
		return color[1], color[2], color[3]
	end
end


function PitBull4_VengeanceBar:PLAYER_SPECIALIZATION_CHANGED()
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM")
	ShowVengeance = nil

	if IsTank() then
		ShowVengeance = true
		MaxVengeanceAP = 1

		self:RegisterEvent("UNIT_AURA")
		self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
		self:UPDATE_SHAPESHIFT_FORM()
	end

	self:UpdateForUnitID("player")
end

function PitBull4_VengeanceBar:UNIT_AURA(_, unit)
	if unit == "player" then
		self:UpdateForUnitID("player")
	end
end

function PitBull4_VengeanceBar:UPDATE_SHAPESHIFT_FORM()
	if self:GetLayoutDB("Player").check_stance then
		ShowVengeance = IsInTankStance()
		self:UpdateForUnitID("player")
	end
end


PitBull4_VengeanceBar:SetLayoutOptionsFunction(function(self)
	return "check_stance", {
		type = "toggle",
		name = L["Hide in non-tank stance"],
		desc = L["Hide the vengeance bar if the player is not in a tanking stance."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).check_stance
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).check_stance = value
			if IsTank() then
				if value then
					ShowVengeance = IsInTankStance()
				else
					ShowVengeance = true
				end
			else
				ShowVengeance = nil
			end
			PitBull4_VengeanceBar:UpdateForUnitID("player")
		end,
		disabled = function()
			return not PitBull4.Options.GetLayoutDB(self).enabled
		end,
	}
end)

